/**
 * Copyright (c) 2002 Bright Side Factory.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Bright Side Factory (http://www.bs-factory.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Bright Side", "BS Factory" and "Bright Side Factory" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact info@bs-factory.org.
 *
 * 5. Products derived from this software may not be called "Bright Side",
 *    nor may "Bright Side" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Bright Side Factory.  For more
 * information on the Bright Side Factory, please see
 * <http://www.bs-factory.org/>.
 */
package com.bsf.lovDemo.components;

import com.bsf.lovDemo.util.BrowserLauncher;
import com.jgoodies.forms.factories.Borders;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.swing.AbstractDialog;
import com.jgoodies.swing.panels.GradientBackgroundPanel;

import javax.swing.JComponent;
import javax.swing.JEditorPane;
import javax.swing.JPanel;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import java.awt.Frame;
import java.net.URL;

/**
 * This class shows a dialog containing a link to the bs-factory website.
 * Usually displayed when the user quits the application (so its name :o).
 */
public class ExitDialog extends AbstractDialog {
    public ExitDialog( Frame owner ) {
        super( owner );
    }

    protected JComponent buildContent() {
        JEditorPane html = null;
        GradientBackgroundPanel background = new GradientBackgroundPanel( false );

        try {
            URL url = null;
            String path = null;
            path = "/docs/bye.html";
            url = getClass().getResource( path );

            if ( url != null ) {
                html = new JEditorPane( url );
                html.setEditable( false );
                html.setOpaque( false );
                html.addHyperlinkListener( new HyperlinkListener() {
                    public void hyperlinkUpdate( HyperlinkEvent e ) {
                        if ( e.getEventType() == HyperlinkEvent.EventType.ACTIVATED ) {
                            BrowserLauncher.openURL( e.getURL().toString() );
                        }
                    }
                } );
            }
        } catch( Exception e ) {
            throw new RuntimeException( e.getLocalizedMessage() );
        }

        FormLayout layout = new FormLayout( "left:500px:grow", "b:250px:grow" );

        JPanel panel = new JPanel( layout );
        panel.setOpaque( false );
        panel.setBorder( Borders.DLU14_BORDER );

        CellConstraints cc = new CellConstraints();

        panel.add( html, cc.xy( 1, 1 ) );
        background.add( panel );

        return background;
    }

    /**
     * Overload the show() method so that the Dialog disposes
     * itself after a 5 secondes timeout
     */
    public void show() {
        Thread closeEndindDialogThread = new Thread() {
            public void run() {
                try {
                    sleep( 5000 );
                } catch( InterruptedException e ) {
                    // Doing nothing if it fails...
                }

                dispose();

                System.exit( 0 );
            }
        };

        closeEndindDialogThread.start();

        super.show();
    }
}
