/**
 * Copyright (c) 2002 Bright Side Factory.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Bright Side Factory (http://www.bs-factory.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Bright Side", "BS Factory" and "Bright Side Factory" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact info@bs-factory.org.
 *
 * 5. Products derived from this software may not be called "Bright Side",
 *    nor may "Bright Side" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Bright Side Factory.  For more
 * information on the Bright Side Factory, please see
 * <http://www.bs-factory.org/>.
 */
package com.bsf.lovDemo.components;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.swing.AbstractDialog;
import com.jgoodies.swing.application.ResourceManager;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * A default dialog to display info message to the user.
 */
public class InfoDialog extends AbstractDialog {
    private static final String EMPTY_STRING = "";

    private JLabel _messageLabel = new JLabel( "", JLabel.CENTER );
    private JLabel _localizedExceptionLabel = new JLabel( "", JLabel.CENTER );

    private JButton _okButton = new JButton( ResourceManager.getString( "dialog.button.ok" ) );

    /**
     * Instantiate a new InfoDialog.
     *
     * @param p_owner The frame owning this dialog if any.
     * @param p_closeOperation The close operation to use (cf. JDialog)
     */
    public InfoDialog( Frame p_owner, int p_closeOperation ) {
        super( p_owner );

        setDefaultCloseOperation( p_closeOperation );

        // We close the dialog when the user presses the button
        _okButton.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                // We should find a way to get this handling to swing
                int closeOperation = getDefaultCloseOperation();

                switch( closeOperation ) {
                    case ( EXIT_ON_CLOSE ):
                        System.exit( 0 );
                    case ( DISPOSE_ON_CLOSE ):
                        dispose();
                        break;
                    case ( HIDE_ON_CLOSE ):
                        hide();
                        break;
                }
            }
        } );
    }

    protected JComponent buildContent() {
        JPanel contentPanel = new JPanel(
                new FormLayout( "10dlu, center:pref:grow, 10dlu",
                                "5dlu, fill:pref, fill:pref, 8dlu, center:min" )
        );

        CellConstraints cellConstraint = new CellConstraints();
        contentPanel.add( _messageLabel, cellConstraint.xy( 2, 2 ) );
        contentPanel.add( _localizedExceptionLabel, cellConstraint.xy( 2, 3 ) );
        contentPanel.add( _okButton, cellConstraint.xy( 2, 5 ) );

        return contentPanel;
    }

    /**
     * Open the dialog and displays the given message.
     *
     * @param p_message The message to display.
     * @param p_localizedExceptionMessage The message of the exception, can be null.
     *
     * @see #open
     */
    public void openWithMessage( String p_message, String p_localizedExceptionMessage ) {
        _messageLabel.setText( p_message );

        if ( p_localizedExceptionMessage == null ) {
            p_localizedExceptionMessage = EMPTY_STRING;
        }

        _localizedExceptionLabel.setText( p_localizedExceptionMessage );

        // We open the dialog
        open();
    }
}
