/**
 * Copyright (c) 2002 Bright Side Factory.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Bright Side Factory (http://www.bs-factory.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Bright Side", "BS Factory" and "Bright Side Factory" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact info@bs-factory.org.
 *
 * 5. Products derived from this software may not be called "Bright Side",
 *    nor may "Bright Side" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Bright Side Factory.  For more
 * information on the Bright Side Factory, please see
 * <http://www.bs-factory.org/>.
 */
package com.bsf.lovDemo.components;

import com.jgoodies.forms.factories.DefaultComponentFactory;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import javax.swing.ButtonGroup;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import java.awt.event.ActionListener;
import java.util.Enumeration;

/**
 * Brings a panel that enable to select what LovValue field a renderer
 * should select. As a matter of fact displays an array of String and
 * enable a selection in the created group... Could be used for anything
 * that would need this feature... :o)
 */
public class LovFieldSelectPanel extends JPanel {
    private static String COLUMN_SIZE = "left:80dlu:none";
    private static String SEPARATOR = ", ";

    private String _title = null;
    private String[] _fields = null;
    private String[] _fieldsLabel = null;
    private ButtonGroup _buttonGroup = null;
    private JComponent _leftComponent = null;
    private ActionListener _actionListener = null;

    private int _initialSelection = 0;

    public LovFieldSelectPanel( ActionListener p_actionListener, String p_title, String[] p_fields, String[] p_fieldsLabel, int p_initialSelection ) {
        this( p_actionListener, null, p_title, p_fields, p_fieldsLabel, p_initialSelection );
    }

    /**
     * @param p_leftComponent The component displayed on the left part.
     * @param p_actionListener The listener notified of the selection.
     * @param p_title ...
     * @param p_fields ...
     * @param p_fieldsLabel ...
     * @param p_initialSelection ...
     */
    public LovFieldSelectPanel( ActionListener p_actionListener, JComponent p_leftComponent, String p_title, String[] p_fields, String[] p_fieldsLabel, int p_initialSelection ) {
        super();

        _title = p_title;
        _fields = p_fields;
        _fieldsLabel = p_fieldsLabel;
        _leftComponent = p_leftComponent;
        _actionListener = p_actionListener;
        _initialSelection = p_initialSelection;

        createLayout();
    }

    public void createLayout() {
        String columnsSpec = "3dlu";

        if ( _leftComponent != null ) {
            columnsSpec += ", fill:130dlu:none, 20dlu";
        }

        for ( int index = 0 ; index < _fields.length ; index++ ) {
            columnsSpec += SEPARATOR + COLUMN_SIZE + SEPARATOR;
        }

        // Growing space for the separator and final space after the components
        columnsSpec += "left:1dlu:grow";

        FormLayout formLayout = new FormLayout( columnsSpec, "4dlu, top:pref:none, 3dlu, center:pref:none" );

        setOpaque( false );
        setLayout( formLayout );

        _buttonGroup = new ButtonGroup();
        CellConstraints celllConstraint = new CellConstraints();

        // Do we have to add the left component ?
        int startingIndex = 2;

        if ( _leftComponent != null ) {
            startingIndex = 4; // The spots for the left component and "insets"
            add( _leftComponent, celllConstraint.xy( 2, 4 ) );
        }

        JComponent separator = DefaultComponentFactory.getInstance().createSeparator( _title );
        separator.setOpaque( false );

        add( separator, celllConstraint.xywh( 1, 2, _fields.length + startingIndex, 1 ) );

        for ( int index = 0 ; index < _fields.length ; index++ ) {
            JRadioButton radioButton = new JRadioButton( _fieldsLabel[ index ] );
            radioButton.setOpaque( false );
            radioButton.setFocusPainted( false );
            radioButton.addActionListener( _actionListener );
            radioButton.setActionCommand( _fields[ index ] );

            add( radioButton, celllConstraint.xy( index + startingIndex, 4 ) );

            // We add it to the button group (single selection)
            _buttonGroup.add( radioButton );

            if ( index == _initialSelection ) {
                _buttonGroup.setSelected( radioButton.getModel(), true );
            }
        }
    }

    /**
     * Overiden to disable the radio buttons...
     */
    public void setEnabled( boolean p_enabled ) {
        Enumeration enum = _buttonGroup.getElements();

        while ( enum.hasMoreElements() ) {
            ( (JRadioButton) enum.nextElement() ).setEnabled( p_enabled );
        }
    }

    public String getSelectedField() {
        return _buttonGroup.getSelection().getActionCommand();
    }
}
