/**
 * Copyright (c) 2002 Bright Side Factory.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Bright Side Factory (http://www.bs-factory.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Bright Side", "BS Factory" and "Bright Side Factory" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact info@bs-factory.org.
 *
 * 5. Products derived from this software may not be called "Bright Side",
 *    nor may "Bright Side" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Bright Side Factory.  For more
 * information on the Bright Side Factory, please see
 * <http://www.bs-factory.org/>.
 */

package com.bsf.lovDemo.views.internalFrames;

import com.bsf.lovDemo.models.AvailableLovs;
import com.bsf.lovDemo.views.DemoFrame;
import com.jgoodies.forms.factories.DefaultComponentFactory;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.swing.application.ResourceManager;
import com.jgoodies.swing.application.Workbench;
import com.jgoodies.swing.panels.GradientBackgroundPanel;
import com.jgoodies.swing.panels.SimpleInternalFrame;
import com.jgoodies.swing.util.UIFactory;
import org.bsf.listOfValues.client.LovManager;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;

/**
 * Provides the InternalFrame for the performance illustration.
 */
public final class PerformancesInternalFrame extends SimpleInternalFrame implements ActionListener {
    private JRadioButton _smallLov = null;
    private JRadioButton _largeLov = null;

    private JButton _retrieveWithoutCache = null;
    private JButton _retrieveWithClientCache = null;
    private JButton _retrieveWithServerCache = null;

    private JLabel _noCacheResultLabel = null;
    private JLabel _serverCacheResultLabel = null;
    private JLabel _clientCacheResultLabel = null;

    private Long _selectedLovOID = null;

    // *************************************************************************
    // ***************************** Constructors ******************************
    // *************************************************************************

    /**
     * Constructs a module displaying the usage of LOVs in a Table.
     */
    public PerformancesInternalFrame() {
        super( ResourceManager.getString( "performances.title" ) );

        // We want this interfnal frame to appear selected
        setSelected( true );

        // We set the content of this internal frame
        setContent( buildContentPanel() );
    }

    // *************************************************************************
    // *************************** Design Methods ******************************
    // *************************************************************************

    /**
     * Builds and returns the content panel.
     */
    private JComponent buildContentPanel() {
        JComponent lovSelectionSeparator = DefaultComponentFactory.getInstance().createSeparator( ResourceManager.getString( "lov.selection.separator" ) );
        lovSelectionSeparator.setOpaque( false );

        _smallLov = new JRadioButton( ResourceManager.getString( "lov.selection.small" ) );
        _smallLov.setOpaque( false );
        _smallLov.addActionListener( this );

        _largeLov = new JRadioButton( ResourceManager.getString( "lov.selection.large" ) );
        _largeLov.setOpaque( false );
        _largeLov.addActionListener( this );

        ButtonGroup buttonGroup = new ButtonGroup();
        buttonGroup.add( _smallLov );
        buttonGroup.add( _largeLov );

        // Default selection (small lov ie. countries)
        _selectedLovOID = AvailableLovs.LOV_COUNTRIES;
        buttonGroup.setSelected( _smallLov.getModel(), true );

        JComponent lovNoCacheSeparator = DefaultComponentFactory.getInstance().createSeparator( ResourceManager.getString( "lov.no.cache" ) );
        lovNoCacheSeparator.setOpaque( false );

        _retrieveWithoutCache = TableInternalFrame.getButton( ResourceManager.getString( "retrieve.button.label" ) );
        _retrieveWithoutCache.addActionListener( this );
        _noCacheResultLabel = getResultLabel();

        JComponent lovServerCacheSeparator = DefaultComponentFactory.getInstance().createSeparator( ResourceManager.getString( "lov.server.cache" ) );
        lovServerCacheSeparator.setOpaque( false );

        _retrieveWithServerCache = TableInternalFrame.getButton( ResourceManager.getString( "retrieve.button.label" ) );
        _retrieveWithServerCache.addActionListener( this );
        _serverCacheResultLabel = getResultLabel();

        JComponent lovClientCacheSeparator = DefaultComponentFactory.getInstance().createSeparator( ResourceManager.getString( "lov.client.cache" ) );
        lovClientCacheSeparator.setOpaque( false );

        _retrieveWithClientCache = TableInternalFrame.getButton( ResourceManager.getString( "retrieve.button.label" ) );
        _retrieveWithClientCache.addActionListener( this );
        _clientCacheResultLabel = getResultLabel();

        GradientBackgroundPanel enclosingPanel = new GradientBackgroundPanel( true );
        enclosingPanel.setLayout( new FormLayout( "4dlu, 20dlu, fill:pref:none, 10dlu, left:pref:none, left:1dlu:grow, 4dlu",
                                                  "3dlu, top:pref:none, 6dlu, center:pref:none, " +
                                                  "12dlu, top:pref:none, 6dlu, center:pref:none, " +
                                                  "12dlu, top:pref:none, 6dlu, center:pref:none, " +
                                                  "12dlu, top:pref:none, 6dlu, center:pref:none, 1dlu:grow" ) );

        CellConstraints cellConstraints = new CellConstraints();
        enclosingPanel.add( lovSelectionSeparator, cellConstraints.xywh( 2, 2, 5, 1 ) );
        enclosingPanel.add( _smallLov, cellConstraints.xy( 3, 4 ) );
        enclosingPanel.add( _largeLov, cellConstraints.xy( 5, 4 ) );
        enclosingPanel.add( lovNoCacheSeparator, cellConstraints.xywh( 2, 6, 5, 1 ) );
        enclosingPanel.add( _retrieveWithoutCache, cellConstraints.xy( 3, 8 ) );
        enclosingPanel.add( _noCacheResultLabel, cellConstraints.xywh( 5, 8, 2, 1 ) );
        enclosingPanel.add( lovServerCacheSeparator, cellConstraints.xywh( 2, 10, 5, 1 ) );
        enclosingPanel.add( _retrieveWithServerCache, cellConstraints.xy( 3, 12 ) );
        enclosingPanel.add( _serverCacheResultLabel, cellConstraints.xywh( 5, 12, 2, 1 ) );
        enclosingPanel.add( lovClientCacheSeparator, cellConstraints.xywh( 2, 14, 5, 1 ) );
        enclosingPanel.add( _retrieveWithClientCache, cellConstraints.xy( 3, 16 ) );
        enclosingPanel.add( _clientCacheResultLabel, cellConstraints.xywh( 5, 16, 2, 1 ) );

        return UIFactory.createStrippedScrollPane( enclosingPanel );
    }

    // *************************************************************************
    // ************************** Action Methods *******************************
    // *************************************************************************

    /**
     * @return A formatted label to display the results.
     */
    protected JLabel getResultLabel() {
        JLabel aLabel = new JLabel( ResourceManager.getString( "not.yet.executed" ) );

        aLabel.setOpaque( false );
        aLabel.setFont( aLabel.getFont().deriveFont( 12.0f ) );
        aLabel.setIcon( ResourceManager.getIcon( "forward.icon" ) );

        return aLabel;
    }

    protected void retrieveWithoutCache() {
        try {
            long time = System.currentTimeMillis();

            // We clear the LOV from the client and the server caches...
            LovManager.resynchronizeLov( _selectedLovOID );

            // And we retrieve it...
            List lov = LovManager.getListOfValues( _selectedLovOID );
            time = System.currentTimeMillis() - time;

            _noCacheResultLabel.setText( "A LOV containing " + lov.size() + " items has been retrieved from the server in " + time + " ms !!!" );
        } catch( Exception e ) {
            // Display error message
            _noCacheResultLabel.setText( "Error in LOV retrieval..." );
        }
    }

    protected void retrieveFromServerCache() {
        try {
            // We clear the LOV from the client cache...
            LovManager.clearLocalCache( _selectedLovOID );

            long time = System.currentTimeMillis();

            // And we retrieve it...
            List lov = LovManager.getListOfValues( _selectedLovOID );
            time = System.currentTimeMillis() - time;

            _serverCacheResultLabel.setText( "A LOV containing " + lov.size() + " items has been retrieved from the server in " + time + " ms !!!" );
        } catch( Exception e ) {
            // Display error message
            _serverCacheResultLabel.setText( "Error in LOV retrieval..." );
        }
    }

    protected void retrieveFromLocalCache() {
        try {
            long time = System.currentTimeMillis();

            // We retrieve the LOV... It is already in cache (but errors...)
            List lov = LovManager.getListOfValues( _selectedLovOID );
            time = System.currentTimeMillis() - time;

            _clientCacheResultLabel.setText( "A LOV containing " + lov.size() + " items has been retrieved from the cache in less than " + ( time + 1 ) + " ms !!!" );
        } catch( Exception e ) {
            // Display error message
            _clientCacheResultLabel.setText( "Error in LOV retrieval..." );
        }
    }

    /**
     * This action performed is called by the various components... Would be
     * better handled by a real world application. Done this way to display
     * all the behaviour in the same file.
     */
    public void actionPerformed( ActionEvent e ) {
        ( (DemoFrame) ( Workbench.getMainFrame() ) ).setMouseCursorBusy();

        Object source = e.getSource();

        try {
            if ( source == _smallLov ) {
                _selectedLovOID = AvailableLovs.LOV_COUNTRIES;
            } else if ( source == _largeLov ) {
                _selectedLovOID = AvailableLovs.LOV_AUTHORS;
            } else if ( source == _retrieveWithoutCache ) {
                retrieveWithoutCache();
            } else if ( source == _retrieveWithServerCache ) {
                retrieveFromServerCache();
            } else if ( source == _retrieveWithClientCache ) {
                retrieveFromLocalCache();
            }
        } finally {
            ( (DemoFrame) ( Workbench.getMainFrame() ) ).setMouseCursorIdle();
        }
    }
}
